/*
 * Camera.cpp
 *
 *  Created on: 28.08.2011
 *      Author: Manuel
 */


#include "Camera.h"

namespace vormd {

Camera::Camera() {
	//this->view_position.set(0.0f,0.0f,120.0f*1.5f);
	this->view_position.set(0.0f,0.0f,4.0f);
	this->view_target.set(0.0f,0.0f,0.0f);
	this->up_direction.set(0.0f, 1.0f, 0.0f);

	yaw = yaw_backup = 270.0f;
	pitch = pitch_backup = 0.0f;

}

Camera::Camera(cml::vector3f vpos,cml::vector3f vtarget)
{

	float x = vpos[0];
	float y = vpos[1];
	float z = vpos[2];
	this->view_position.set(x,y,z);
	x = vtarget[0];
	y = vtarget[1];
	z = vtarget[2];
	this->view_target.set(x,y,z);
	this->up_direction.set(0.0f, 1.0f, 0.0f);


	//LOGI("one %f,two %f, three %f",view_position[0],view_position[1],view_position[2]);
	//LOGI("one %f,two %f, three %f",view_target[0],view_target[1],view_target[2]);

	yaw = yaw_backup = 270.0f;
	pitch = pitch_backup = 0.0f;
}

Camera::~Camera() {
	// TODO Auto-generated destructor stub
}

void Camera::setPerspective(float width, float height, float zNear, float zFar)
{
	cml::matrix_perspective_yfov_RH(projection,45.0f,float(width/height),zNear,zFar,cml::z_clip_neg_one);
	cml::matrix_orthographic_RH(orthoprojection,-1.0f,1.0f,-1.0f,1.0f,zNear,zFar,cml::z_clip_neg_one);
}


cml::matrix44f_c Camera::getPerspective()
{
	return projection;
}

cml::matrix44f_c Camera::getOrthoPerspective()
{
	return orthoprojection;
}

cml::matrix44f_c Camera::getView()
{
	return view;
}

cml::matrix44f_c Camera::getVP()
{
	float cosYaw = cosf(cml::rad(yaw));
	float sinYaw = sinf(cml::rad(yaw));
	float sinPitch = sinf(cml::rad(pitch));
	    // calculating the target we are looking at based on our new camera position and the current yaw and pitch
	view_target[0] = view_position[0] + cosYaw;
	view_target[1] = view_position[1] + sinPitch;
	view_target[2] = view_position[2] + sinYaw;

	cml::matrix44f_c vp = cml::identity_4x4();
	cml::matrix_look_at(view,view_position,view_target,up_direction,cml::right_handed);
	vp = projection*view;
	return vp;
}

cml::matrix44f_c Camera::getVOP()
{
	float cosYaw = cosf(cml::rad(yaw));
	float sinYaw = sinf(cml::rad(yaw));
	float sinPitch = sinf(cml::rad(pitch));

	// calculating the target we are looking at based on our new camera position and the current yaw and pitch

	view_target[0] = view_position[0] + cosYaw;
	view_target[1] = view_position[1] + sinPitch;
	view_target[2] = view_position[2] + sinYaw;


	cml::matrix44f_c vop = cml::identity_4x4();
	cml::matrix_look_at(view,view_position,view_target,up_direction,cml::right_handed);
	vop = orthoprojection*view;
	return vop;
}

void Camera::zoom(float zoom)
{
	 float cosYaw = cosf(cml::rad(yaw));
	 float sinYaw = sinf(cml::rad(yaw));
	 view_position[0] += float(cosYaw)*zoom;//*speed;
	 view_position[2] += float(sinYaw)*zoom;//*speed;
}



void Camera::rotate(cml::matrix44f_c rotate)
{
	/*yaw -= 1.0f;
	if (yaw >= 360.0f) yaw -= 360.0f;
	if (yaw < 0.0f) yaw += 360.0f;*/

	cml::vector4f pos = cml::vector4f(view_position[0],view_position[1],view_position[2],1.0f);
	pos =rotate*pos;
	view_position[0] = pos[0];
	view_position[1] = pos[1];
	view_position[2] = pos[2];
}

} /* namespace vormd */
