/*
 * JNIInterface.cpp
 *
 *  Created on: 03.12.2011
 *  Author: Manuel Hochmayr, e0627715@student.tuwien.ac.at
 *
 *  The class is the interface between java and c++. It handles over all the UI input which is done in java to c++
 */

#include <jni.h>
#include <android/bitmap.h>
#include <sys/types.h>
#include <assert.h>
#include <android/asset_manager.h>
#include <android/asset_manager_jni.h>
#include "Renderer.h"


namespace vormd {

bool initialized = false;
bool reload = false;
bool fileLoaded;
Renderer *renderer;
Volume *volume;
cml::matrix44f_c safeVolumeMatrix;
cml::matrix44f_c scaleMatrix;

extern "C" {
JNIEXPORT jboolean JNICALL Java_com_vormd_VoRMDJNILIB_assetloadFile(JNIEnv* env,jobject obj, jobject assetManager, jstring filename);
JNIEXPORT jboolean JNICALL Java_com_vormd_VoRMDJNILIB_loadFile(JNIEnv* env,jobject obj, jstring filename);
JNIEXPORT jboolean JNICALL Java_com_vormd_VoRMDJNILIB_loadTransferPNG(JNIEnv* env,jobject obj, jobject bitmap,jboolean usingSTF);
JNIEXPORT void JNICALL Java_com_vormd_VoRMDJNILIB_init(JNIEnv * env, jobject obj, jint width, jint height);
JNIEXPORT void JNICALL Java_com_vormd_VoRMDJNILIB_draw(JNIEnv * env, jobject obj, jint mode);
JNIEXPORT void JNICALL Java_com_vormd_VoRMDJNILIB_rotateLeft(JNIEnv * env,jobject obj);
JNIEXPORT void JNICALL Java_com_vormd_VoRMDJNILIB_rotateRight(JNIEnv * env,jobject obj);
JNIEXPORT void JNICALL Java_com_vormd_VoRMDJNILIB_rotateUp(JNIEnv * env,jobject obj);
JNIEXPORT void JNICALL Java_com_vormd_VoRMDJNILIB_rotateDown(JNIEnv * env,jobject obj);
JNIEXPORT void JNICALL Java_com_vormd_VoRMDJNILIB_scale(JNIEnv * env,jobject obj, jfloat);
JNIEXPORT jfloat JNICALL Java_com_vormd_VoRMDJNILIB_getFPS(JNIEnv * env,jobject obj);
JNIEXPORT jint JNICALL Java_com_vormd_VoRMDJNILIB_getVolumeDepth(JNIEnv * env,jobject obj);
JNIEXPORT jint JNICALL Java_com_vormd_VoRMDJNILIB_getVolumeHeight(JNIEnv * env,jobject obj);
JNIEXPORT jint JNICALL Java_com_vormd_VoRMDJNILIB_getVolumeWidth(JNIEnv * env,jobject obj);
JNIEXPORT void JNICALL Java_com_vormd_VoRMDJNILIB_setZCoord(JNIEnv * env,jobject obj, jint zcoord);
JNIEXPORT void JNICALL Java_com_vormd_VoRMDJNILIB_setXCoord(JNIEnv * env,jobject obj, jint zcoord);
JNIEXPORT void JNICALL Java_com_vormd_VoRMDJNILIB_setYCoord(JNIEnv * env,jobject obj, jint zcoord);
JNIEXPORT void JNICALL Java_com_vormd_VoRMDJNILIB_performingUpdate(JNIEnv * env,jobject obj, jboolean update);
JNIEXPORT void JNICALL Java_com_vormd_VoRMDJNILIB_setDz(JNIEnv * env,jobject obj, jfloat jdz);
JNIEXPORT void JNICALL Java_com_vormd_VoRMDJNILIB_setAdaptiv(JNIEnv * env,jobject obj, jboolean adapt);
JNIEXPORT void JNICALL Java_com_vormd_VoRMDJNILIB_setSlicingMode(JNIEnv * env,jobject obj, jint coord);
JNIEXPORT void JNICALL Java_com_vormd_VoRMDJNILIB_quitApp(JNIEnv * env,jobject obj);

};

/**
 * Method which initializes the renderer. If the activity was running before it will take the saved volume matrix
 */
void JNICALL Java_com_vormd_VoRMDJNILIB_init(JNIEnv * env, jobject obj,jint width, jint height) {

	if(!initialized){
		renderer = new Renderer();
		renderer->initNDK(width, height);
		initialized = true;
	}else{
		safeVolumeMatrix =volume->volumeModelMatrix;
		renderer = new Renderer();
		renderer->initNDK(width, height);
		reload = true;
	}
}

/*
 * Will be called for the java render loop to call the draw mehtod of the c++ renderer. It also handels the render modes and
 * checks wheter the volume rendering must be updated or not.
 */
void JNICALL Java_com_vormd_VoRMDJNILIB_draw(JNIEnv * env, jobject obj,jint mode) {

	bool modeChanged = false;

	if(renderer->rendermode != mode ) modeChanged = true;

	renderer->fpsCalc();


	if (renderer->rendermode == -1) {
		renderer->rendermode = mode;
		if (mode == Renderer::SLICING) {
			renderer->drawSlices();
		} else {
			renderer->performUpdate = true;
			renderer->update();
			renderer->performUpdate = false;
		}
	} else {
		renderer->rendermode = mode;

		if (mode == Renderer::SLICING) {
			renderer->drawSlices();
		} else {
			if(modeChanged){
				renderer->update();
			}
			if (renderer->adaptivSampling) {
				if (renderer->performUpdate || renderer->buttondzPressed) {
					renderer->update();
					renderer->fromUpdate = true;
					renderer->buttondzPressed = false;
				} else {
					if (renderer->fromUpdate) {
						renderer->update();
						renderer->fromUpdate = false;
					} else {
						renderer->draw();
					}
				}
			} else {
				renderer->update();
			}
		}
	}

	if (mode != Renderer::SLICING) {
		if(renderer->performUpdate || renderer->buttondzPressed) {
			renderer->drawFBOOrientationCube();
		}else{
			if(renderer->fromUpdate){
				renderer->drawFBOOrientationCube();
			}else{
				renderer->drawOrientationCube();
			}
		}
	}


}
/**
 * Will be called when the touchinput is pressed and when the touch ends
 */
void JNICALL Java_com_vormd_VoRMDJNILIB_performingUpdate(JNIEnv * env,jobject obj, jboolean update) {
	renderer->performUpdate = update;
}
/*
 * Rotation methods
 */
void JNICALL Java_com_vormd_VoRMDJNILIB_rotateLeft(JNIEnv * env, jobject obj) {
	cml::matrix_rotate_about_world_y(volume->volumeModelMatrix, cml::rad(1.5f));
	//LOGI("C++ rotLeft");
}

void JNICALL Java_com_vormd_VoRMDJNILIB_rotateRight(JNIEnv * env, jobject obj) {
	cml::matrix_rotate_about_world_y(volume->volumeModelMatrix, cml::rad(-1.5f));
	//LOGI("C++ rotRight");
}

void JNICALL Java_com_vormd_VoRMDJNILIB_rotateUp(JNIEnv * env, jobject obj) {
	cml::matrix_rotate_about_world_x(volume->volumeModelMatrix, cml::rad(-1.5f));
	//LOGI("C++ rotUp");
}

void JNICALL Java_com_vormd_VoRMDJNILIB_rotateDown(JNIEnv * env, jobject obj) {
	cml::matrix_rotate_about_world_x(volume->volumeModelMatrix, cml::rad(1.5f));
	//LOGI("C++ rotDown");
}
/*
 * Scaling
 */
void JNICALL Java_com_vormd_VoRMDJNILIB_scale(JNIEnv * env, jobject obj,jfloat scalefactor) {
	float scf = scalefactor;
	cml::matrix_scale(scaleMatrix, scf, scf, scf);
	renderer->scale = scaleMatrix;
}

/**
 * Returns the fps to java
 */
jfloat JNICALL Java_com_vormd_VoRMDJNILIB_getFPS(JNIEnv * env, jobject obj) {
	return renderer->fps;
}


jint JNICALL Java_com_vormd_VoRMDJNILIB_getVolumeDepth(JNIEnv * env,jobject obj) {
	return volume->GetDepth();

}

jint JNICALL Java_com_vormd_VoRMDJNILIB_getVolumeHeight(JNIEnv * env,jobject obj){
	return volume->GetHeight();
}

jint JNICALL Java_com_vormd_VoRMDJNILIB_getVolumeWidth(JNIEnv * env,jobject obj){
	return volume->GetWidth();
}

void JNICALL Java_com_vormd_VoRMDJNILIB_setZCoord(JNIEnv * env, jobject obj,jint zcoord) {
	renderer->zCoord = (float) zcoord / (float) volume->GetHeight();
	//LOGI("zCoord: %f", renderer->zCoord);
}

void JNICALL Java_com_vormd_VoRMDJNILIB_setXCoord(JNIEnv * env, jobject obj,jint xcoord) {
	renderer->zCoord = (float) xcoord / (float) volume->GetHeight();
	//LOGI("zCoord: %f", renderer->zCoord);
}

void JNICALL Java_com_vormd_VoRMDJNILIB_setYCoord(JNIEnv * env, jobject obj,jint ycoord) {
	renderer->zCoord = (float) ycoord / (float) volume->GetHeight();
	//LOGI("zCoord: %f", renderer->zCoord);
}

void JNICALL Java_com_vormd_VoRMDJNILIB_setDz(JNIEnv * env,jobject obj, jfloat jdz){
	renderer->dz = (float)jdz;
	renderer->buttondzPressed = true;
}

/**
 * Adaptive sampling on/off
 */
void JNICALL Java_com_vormd_VoRMDJNILIB_setAdaptiv(JNIEnv * env,jobject obj, jboolean adapt){
	renderer->adaptivSampling = adapt;
}

/**
 * Changing the slicing mode
 */
void JNICALL Java_com_vormd_VoRMDJNILIB_setSlicingMode(JNIEnv * env,jobject obj, jint coord){
	if(coord == renderer->X){
		renderer->slicingMode = renderer->X;
	}

	if(coord == renderer->Y){
			renderer->slicingMode = renderer->Y;
	}
	if(coord == renderer->Z){
			renderer->slicingMode = renderer->Z;
	}
}

/**
 * Method to load a file form the asset folde. Assets can have max of 1.5 MB. So only the lobster is delivered with the program.
 * The rest is loaded from SD card by using a normal c++ file loader.
 */
jboolean JNICALL Java_com_vormd_VoRMDJNILIB_assetloadFile(JNIEnv* env, jobject obj,jobject assetManager, jstring filename)
{
	if(fileLoaded){
		volume->~Volume();
		fileLoaded = false;
	}

	volume = new Volume();
	// convert Java string to UTF-8
	const jbyte* utf8 = (jbyte*) env->GetStringUTFChars(filename, NULL);
	assert(NULL != utf8);
	AAssetManager* mgr = AAssetManager_fromJava(env, assetManager);

	assert(NULL != mgr);
	AAsset* asset = AAssetManager_open(mgr, (const char*) utf8,
			AASSET_MODE_UNKNOWN);
	// release the Java string and UTF-8
	env->ReleaseStringUTFChars(filename, (const char *) utf8);

	// the asset might not be found
	if (asset == NULL) {
		LOGE("FILE NOT FOUND");
		return JNI_FALSE;
	}

	// open asset as file descriptor
	off_t start, length;
	int fd = AAsset_openFileDescriptor(asset, &start, &length);
	if (0 <= fd){
		LOGE("File compressed");
	}
	else{
		LOGI("FILE NOT COMPRESSED");

		unsigned short uWidth[1], uHeight[1], uDepth[1];

		off_t remain = AAsset_getRemainingLength(asset);
		LOGI("Bytes remaining: %d", remain);

		AAsset_read(asset, &uWidth, sizeof(unsigned short));
		AAsset_read(asset, &uHeight, sizeof(unsigned short));
		AAsset_read(asset, &uDepth, sizeof(unsigned short));

		volume->m_iWidth = int(uWidth[0]);
		volume->m_iHeight = int(uHeight[0]);
		volume->m_iDepth = int(uDepth[0]);

		remain = AAsset_getRemainingLength(asset);
		LOGI("Bytes remaining: %d", remain);

		const int iSlice = volume->m_iWidth * volume->m_iHeight;
		const int iSize = iSlice * volume->m_iDepth;
		volume->m_vecVoxels.resize(iSize);

		std::vector<unsigned short> vecData;
		vecData.resize(iSize);

		LOGI("SIZE: %d", vecData.size());

		unsigned short value;
		for (int i = 0; i < iSize; i++) {

			AAsset_read(asset, &value, sizeof(unsigned short));
			vecData[i] = value;

		}

		remain = AAsset_getRemainingLength(asset);
			LOGI("Bytes remaining: %d", remain);

			for (int k = 0; k < volume->m_iDepth; k++) {
				for (int j = 0; j < volume->m_iHeight; j++) {
					for (int i = 0; i < volume->m_iWidth; i++) {
						//we convert the data to float values in an interval of [0..1]
						float fValue = std::min(1.0f,float(vecData[i + j * volume->m_iWidth+ k * iSlice]) / 4095.0f);
							volume->m_vecVoxels[(i + j * volume->m_iWidth + k * iSlice)] = Voxel(static_cast<unsigned char>(fValue * 255.0f));
					}
				}
			}

	volume->CreateTexture3D();
	AAsset_close(asset);

	if(reload){
		volume->volumeModelMatrix = safeVolumeMatrix;
	}
	renderer->SetVolume(volume);

	fileLoaded = true;

	return JNI_TRUE;
	}

	return JNI_FALSE;
}

/**
 * Mehtod to load a file from the SD card
 */
jboolean JNICALL Java_com_vormd_VoRMDJNILIB_loadFile(JNIEnv* env, jobject obj,jstring filename)
{

	const char *nativeString = env->GetStringUTFChars(filename, 0);


	if(fileLoaded){
		delete volume;
		fileLoaded = false;
	}

	volume = new Volume();

	if(volume->LoadFile(nativeString))
	{
		renderer->SetVolume(volume);
		fileLoaded = true;
		return JNI_TRUE;
	}else{
		return JNI_FALSE;
	}

	return JNI_FALSE;
}

/**
 * Method to load a PNG image
 */
jboolean JNICALL Java_com_vormd_VoRMDJNILIB_loadTransferPNG(JNIEnv* env,jobject obj, jobject bitmap,jboolean usingStandard)
{
	if(usingStandard){
		renderer->GetTransferClass()->createStandardTransferFunction();

	}else{
		AndroidBitmapInfo  info;
		unsigned char *pixels;
		int ret;

		AndroidBitmap_getInfo(env, bitmap, &info);

		if(info.format != ANDROID_BITMAP_FORMAT_RGBA_8888) {
		  LOGE("Bitmap format is not RGBA_8888!");
		  return false;
		}else{
			AndroidBitmap_lockPixels(env, bitmap, reinterpret_cast<void **>(&pixels));
			renderer->GetTransferClass()->LoadTransferTexture(info.width,pixels);
			AndroidBitmap_unlockPixels(env, bitmap);
			return true;
		}
	}
	return false;

}

/**
 * Will be called when the quit button was pressed
 */
void JNICALL Java_com_vormd_VoRMDJNILIB_quitApp(JNIEnv * env,jobject obj)
{
	safeVolumeMatrix = cml::identity_4x4();
	scaleMatrix = cml::identity_4x4();
	renderer->~Renderer();
}

}

