/*
 * Shadermanager.cpp
 *
 *  Created on: 17.08.2011
 *      Author: Manuel
 */

#include "Shadermanager.h"

namespace vormd {

static const char gCubeVertexShader[] =

    "attribute vec4 vPosition;\n"
	"attribute vec4 vcolor;\n"
	"varying vec4 color; \n"
	"uniform mat4 mvp;\n"
    "void main() {\n"
	"	color = vcolor;\n"
    "	gl_Position = mvp*vPosition;\n"
	""
    "}\n";

static const char gCubeFragmentShader[] =
    "precision mediump float;\n"
	"varying vec4 color; \n"
    "void main() {\n"
    "  gl_FragColor = color;\n"
    "}\n";


static const char gPlaneVertexShader[] =
    "attribute vec4 vPosition;\n"
	"attribute vec2 vertexTexCoord;\n"
	"varying vec2 texCoord\n;"
	"uniform mat4 mvp;\n"
    "void main() {\n"
	"   texCoord = vertexTexCoord;"
    "	gl_Position = mvp*vPosition;\n"
	""
    "}\n";

static const char gPlaneFragmentShader[] =
    "precision mediump float;\n"
	"uniform sampler2D tex_entry;\n"
	"varying vec2 texCoord; \n"
    "void main() {\n"
    "  gl_FragColor = texture2D(tex_entry, texCoord);\n"
    "}\n";




static const char SlicesVertexShader[] =
{
	"precision highp float; \n"
    "attribute vec4 vPosition;\n"
	"attribute vec2 vertexTexCoord;\n"
	"varying vec2 texCoord\n;"
	"uniform mat4 mvp;\n"
    "void main() {\n"
	"	gl_Position = mvp*vPosition;\n"
	"   texCoord = vertexTexCoord;\n"
	""
    "}\n"
};

static const char SlicesFragShader[] =
 {
   "#extension GL_OES_texture_3D:enable \n"
	"precision highp float; \n"
	"uniform sampler3D tex_intensity;\n"
	"varying vec2 texCoord;\n"
	"uniform float zCoord;"
	"uniform vec3 textureScale; \n"
	"uniform int slicingmode;"

		 "void main()\n"
		 "{\n"

		 	 "vec3 texCoordS;\n"
		 	 "vec4 intensity;\n"

		 	 //X
		 	 "if(slicingmode == 0){\n"
				"texCoordS.x = zCoord* textureScale.x;\n"
				"texCoordS.y = texCoord.x* textureScale.y;\n"
				"texCoordS.z = texCoord.y* textureScale.z;"
				"intensity = texture3D(tex_intensity, vec3(texCoordS.x,texCoordS.y,texCoordS.z));\n"
		 	 "}\n"

		 	 //Y
		 	 "if(slicingmode == 1){\n"
		 		"texCoordS.x = texCoord.x* textureScale.x;\n"
		 		"texCoordS.y = zCoord* textureScale.y;\n"
		 		"texCoordS.z = texCoord.y* textureScale.z;"
		 		"intensity = texture3D(tex_intensity, vec3(texCoordS.x,texCoordS.y,texCoordS.z));\n"
		 	 "}\n"

		 	 //Z
			 "if(slicingmode == 2){\n"
				"texCoordS.x = texCoord.x* textureScale.x;\n"
				"texCoordS.y = texCoord.y* textureScale.y;\n"
				"texCoordS.z = zCoord* textureScale.z;"
				"intensity = texture3D(tex_intensity, vec3(texCoordS.x,texCoordS.y,texCoordS.z));\n"
			 "}\n"


		 	"gl_FragColor = intensity;\n"
		 "}\n"

 };


static const char gRayVertexShader[] =
	"precision highp float; \n"
    "attribute vec4 vPosition;\n"
	"attribute vec2 vertexTexCoord;\n"
	"varying vec2 texCoord\n;"
	"uniform mat4 mvp;\n"
    "void main() {\n"
	"	gl_Position = mvp*vPosition;\n"
	"   texCoord = vertexTexCoord;\n"
	""
    "}\n";

static const char gRayFragmentShader[] =
 {
"#extension GL_OES_texture_3D:enable \n"
"precision highp float;\n"
"uniform sampler3D tex_intensity;\n"
"uniform sampler2D tex_transfer;\n"
"uniform sampler2D tex_entry;\n"
"uniform sampler2D tex_exit;\n"
"uniform sampler2D orientOverlay;\n"
"uniform vec3 textureScale; \n"
"uniform int rm;\n"
"uniform int performUpdate;"
"uniform float dz;\n"
"varying vec2 texCoord;\n"

"const float delta = 0.005;\n"


"vec3 shading(vec3 N, vec3 V, vec3 L, vec3 color)\n"
	"{\n"
		"float ambient = 0.7;\n"
		"float diffuse= 0.8;\n"
		"float specular = 1.0;\n"
		"float specExp = 100.0;\n"

		"vec3 lightColor = vec3(1.0, 1.0, 1.0);\n"
		"vec3 H = normalize(L + V);\n"

		"float diffFactor = max(dot(N, L), 0.0);\n"
		"float specFactor = pow(max(dot(N, H), 0.0), specExp);\n"

		"vec3 result = vec3(specFactor, specFactor, specFactor);\n"
		"result.x = color.x * ambient + (color.x * diffFactor * diffuse) + lightColor.x * (specFactor * specular);\n"
		"result.y = color.y * ambient + (color.y * diffFactor * diffuse) + lightColor.y * (specFactor * specular);\n"
		"result.z = color.z * ambient + (color.z * diffFactor * diffuse) + lightColor.z * (specFactor * specular);\n"

		"return result;\n"
	" }\n"


"void main()\n"
"{\n"

	"vec4 front = texture2D(tex_entry, texCoord);\n"
	"vec4 orientationcol = texture2D(tex_entry, vec2(0.5,0.5));\n"


	"if (front.w == 0.0)\n"
		 	"discard;\n"

	"vec4 back = texture2D(tex_exit, texCoord);\n"
	"vec3 direction = (back.xyz - front.xyz);\n"
	"vec4 stepSize = vec4(normalize(direction),1.0) * dz;\n"

	"float l = length(direction);"
	"float ls = length(stepSize);"

	"vec4 pos = vec4(front.xyz,0.0);\n"
	"vec4 result = vec4(0.0, 0.0, 0.0, 0.0);\n"
	"vec3 samp1 = vec3(0, 0, 0);\n"
	"vec3 samp2 = vec3(0, 0, 0);\n"

	"vec3 posScale;"

	"int iterCounter = int(ceil(l/ls/128.0));"
	"int maxiter = int(ceil(l/ls));\n"


	"if(iterCounter == 0) iterCounter = 1;"

	"while(pos.w < l){"

		 "for(int i= 0; i < iterCounter;i++){"
			 "posScale.x = pos.x*textureScale.x;"
			 "posScale.y = pos.y*textureScale.y;"
			 "posScale.z = pos.z*textureScale.z;"
			 "vec4 intensity = texture3D(tex_intensity, posScale);\n"

			 //MIP
			 "if(rm == 0){\n"
				 "if(intensity.x > result.x)\n"
					 "result = intensity;\n"
			 "}\n"

			 "else{\n"
				 "samp1.x = texture3D(tex_intensity, posScale.xyz - vec3(delta, 0, 0)).x;\n"
				 "samp2.x = texture3D(tex_intensity, posScale.xyz + vec3(delta, 0, 0)).x;\n"
				 "samp1.y = texture3D(tex_intensity, posScale.xyz - vec3(0, delta, 0)).x;\n"
				 "samp2.y = texture3D(tex_intensity, posScale.xyz + vec3(0, delta, 0)).x;\n"
				 "samp1.z = texture3D(tex_intensity, posScale.xyz - vec3(0, 0, delta)).x;\n"
				 "samp2.z = texture3D(tex_intensity, posScale.xyz + vec3(0, 0, delta)).x;\n"

				 "vec4 transf = texture2D(tex_transfer, vec2(intensity.x,0.0));\n"

				 //Averaging
				 "if(rm == 1){\n"
					 "result.xyz += transf.w*transf.xyz;\n"
					 "result.w += transf.w;\n"
				  "}\n"

				 //Compositing
				 "if(rm == 2){\n"

					 "vec3 N = normalize(samp2 - samp1);\n"
					 "vec3 L = normalize(vec3(0.0, 1.0, 1.0));\n"
					 "vec3 V = vec3(direction);\n"

					 "vec3 shadedColor = shading(N, V, L, transf.xyz);\n"
					 "result.xyz += (1.0 - result.w) * transf.w * shadedColor.xyz;\n"
					 "result.w += (1.0 - result.w) * transf.w;\n"
				 "}\n"

			 "}\n"

			"pos += stepSize;\n"

			 "if (pos.w >= l){"
				 "break;\n"
			 "}"

			"if (result.x >= 1.0)\n"
				"break;\n"

		"}"
	"}\n"
		"if(performUpdate == 1){"
		 	 "gl_FragColor = result+vec4(orientationcol.xyz,0.3);\n"
 	 	 "}else{"
		 	 "if(rm == 1){"
		 	 "	result /= float(maxiter);"
		 	 "	gl_FragColor = result;"
		 	 "}else{"
		 	 "	gl_FragColor = result;"
		 	 "}"
		 "}"
"}\n"

 };





static const char gOrientCubeFragmentShader[] =
 {

	"#extension GL_OES_texture_3D:enable \n"
	"precision highp float;\n"
	"uniform sampler3D tex_intensity;\n"
	"uniform sampler2D tex_transfer;\n"
	"uniform sampler2D tex_entry;\n"
	"uniform sampler2D tex_exit;\n"
	"uniform vec3 textureScale; \n"
	"const float dz = 0.01;\n"
	"varying vec2 texCoord;\n"

	"void main()\n"
	"{\n"

		 "vec4 front = texture2D(tex_entry, texCoord);\n"

		 "vec4 orientationcol = texture2D(tex_entry, vec2(0.5,0.5));\n"

		 "if (front.w == 0.0)\n"
		 	 "discard;\n"

		 	"vec4 back = texture2D(tex_exit, texCoord);\n"
		 	"vec3 direction = (back.xyz - front.xyz);\n"
		 	"vec4 stepSize = vec4(normalize(direction),1.0) * dz;\n"
		 	"int maxiter = int(floor(stepSize));\n"


		 	"float l = length(direction);"
		 	"vec4 pos = vec4(front.xyz,0.0);\n"
		 	"vec4 result = vec4(0.0, 0.0, 0.0, 0.0);\n"
		 	"vec3 posScale;"

		 	"while(pos.w < l){"

		 		"posScale.x = pos.x*textureScale.x;"
		 		"posScale.y = pos.y*textureScale.y;"
		 		"posScale.z = pos.z*textureScale.z;"
		 		"vec4 intensity = texture3D(tex_intensity, posScale);\n"

		 		"if(intensity.x > result.x)\n"
		 				 "result = intensity;\n"


		 		"pos += stepSize;\n"

		 		"if (result.x >= 1.0)\n"
		 			"break;\n"
		 	"}\n"

		 		"gl_FragColor = result+vec4(orientationcol.xyz,0.3);\n"

		 "}\n"
 };


Shadermanager::Shadermanager() {
 this->putShader(0,this->createProgram(gCubeVertexShader,gCubeFragmentShader));
 this->putShader(1,this->createProgram(gPlaneVertexShader,gPlaneFragmentShader));
 this->putShader(2,this->createProgram(gRayVertexShader,gRayFragmentShader));
 this->putShader(3,this->createProgram(SlicesVertexShader,SlicesFragShader));
 this->putShader(4,this->createProgram(gRayVertexShader,gOrientCubeFragmentShader));
}

Shadermanager::~Shadermanager() {
	glDeleteProgram(this->popShader(0)->programmID);
	glDeleteProgram(this->popShader(1)->programmID);
	glDeleteProgram(this->popShader(2)->programmID);
	glDeleteProgram(this->popShader(3)->programmID);
	glDeleteProgram(this->popShader(4)->programmID);

	shaderMap.clear();
}


void Shadermanager::checkGlError(const char* op) {
    for (GLint error = glGetError(); error; error
            = glGetError()) {
        LOGI("after %s() glError (0x%x)\n", op, error);
    }
}

GLuint Shadermanager::loadShader(int shaderType, const char* pSource) {

    GLuint shader = glCreateShader(shaderType);
    if (shader) {
        glShaderSource(shader, 1, &pSource, NULL);
        glCompileShader(shader);
        GLint compiled = 0;
        glGetShaderiv(shader, GL_COMPILE_STATUS, &compiled);
        if (!compiled) {
            GLint infoLen = 0;
            glGetShaderiv(shader, GL_INFO_LOG_LENGTH, &infoLen);
            if (infoLen) {
                char* buf = (char*) malloc(infoLen);
                if (buf) {
                    glGetShaderInfoLog(shader, infoLen, NULL, buf);
                    LOGE("Could not compile shader %d:\n%s\n",
                            shaderType, buf);
                    free(buf);
                }
                glDeleteShader(shader);
                shader = 0;
            }
        }
    }
    return shader;
}

Shader* Shadermanager::createProgram(const char* pVertexSource, const char* pFragmentSource) {
	Shader *shader = new Shader();

	GLuint vertexShader = loadShader(GL_VERTEX_SHADER, pVertexSource);

	if (!vertexShader) {
		shader->programmID =0;
        return shader;
    }

    GLuint fragmentshader = loadShader(GL_FRAGMENT_SHADER, pFragmentSource);
    if (!fragmentshader) {
    	shader->programmID =0;
    	return shader;
    }

    GLuint program = glCreateProgram();
    if (program) {
        glAttachShader(program, vertexShader);
        checkGlError("glAttachShader");
        glAttachShader(program, fragmentshader);
        checkGlError("glAttachShader");
        glLinkProgram(program);
        GLint linkStatus = GL_FALSE;
        glGetProgramiv(program, GL_LINK_STATUS, &linkStatus);
        if (linkStatus != GL_TRUE) {
            GLint bufLength = 0;
            glGetProgramiv(program, GL_INFO_LOG_LENGTH, &bufLength);
            if (bufLength) {
                char* buf = (char*) malloc(bufLength);
                if (buf) {
                    glGetProgramInfoLog(program, bufLength, NULL, buf);
                    LOGE("Could not link program:\n%s\n", buf);
                    free(buf);
                }
            }
            glDeleteProgram(program);
            program = 0;
        }
    }

    shader->vPositionHandle = glGetAttribLocation(program, "vPosition");
    shader->texCoordHandle = glGetAttribLocation(program, "vertexTexCoord");
    shader->MVPMatrixHandle = glGetUniformLocation(program, "mvp");
    shader->textureentryHandle = glGetUniformLocation(program,"tex_entry");
    shader->textureexitHandle = glGetUniformLocation(program,"tex_exit");
    shader->texture3DHandle = glGetUniformLocation(program,"tex_intensity");
    shader->textransferHandle = glGetUniformLocation(program,"tex_transfer");
    shader->vColorHandle = glGetAttribLocation(program, "vcolor");
    shader->zCoordHandle = glGetUniformLocation(program,"zCoord");
    shader->stepSize = glGetUniformLocation(program,"dz");
    shader->textureScale = glGetUniformLocation(program,"textureScale");
    shader->renderMode = glGetUniformLocation(program,"rm");
    shader->perfUpdate = glGetUniformLocation(program,"performUpdate");
    shader->slicingMode = glGetUniformLocation(program,"slicingmode");

    shader->programmID = program;
    return shader;
}

void Shadermanager::putShader(int type, Shader* shader){
	shaderMap[type] = shader;
}

Shader* Shadermanager::popShader(int type){

	return shaderMap[type];
}



} /* namespace vormd */
