/*
 * Volume.cpp
 *
 *  Created on: 17.08.2011
 *      Author: Manuel
 */

#include "Volume.h"

namespace vormd {

const int GetNextPowerOfTwo(const int iNumber) {
	int iPowerOfTwo = 1;

	while (iPowerOfTwo < iNumber)
		iPowerOfTwo = iPowerOfTwo * 2;

	return iPowerOfTwo;
}
;

Volume::Volume() {
	m_iWidth = 1;
	m_iHeight = 1;
	m_iDepth = 1;

	volumeModelMatrix = cml::identity_4x4();

	GLfloat tmpVolumeBoxvertices[] = {
	//front
			-1.00f, -1.00f, 1.00f, //bottom left
			1.00f, -1.00f, 1.00f, //bottom right
			1.00f, 1.00f, 1.00f, //top right
			-1.00f, 1.00f, 1.00f, //top left
			// back
			-1.00f, 1.00f, -1.00f, //bottom left
			-1.00f, -1.00f, -1.00f, //top right
			1.00f, -1.00f, -1.00f, //top right
			1.00f, 1.00f, -1.00f, //top left
			// top
			-1.00f, 1.00f, -1.00f, //bottom right
			-1.00f, 1.00f, 1.00f, //bottom left
			1.00f, 1.00f, 1.00f, //top left
			1.00f, 1.00f, -1.00f, //top right
			// bottom
			-1.00f, -1.00f, -1.00f, //bottom right
			-1.00f, -1.00f, 1.00f, //bottom left
			1.00f, -1.00f, 1.00f, //top left
			1.00f, -1.00f, -1.00f, //top right
			// right
			1.00f, 1.00f, 1.00f, //top left
			1.00f, -1.00f, 1.00f, //bottom right
			1.00f, -1.00f, -1.00f, //top right
			1.00f, 1.00f, -1.00f, //top left
			// left
			-1.00f, 1.00f, 1.00f, //bottom left
			-1.00f, -1.00f, 1.00f, //bottom right
			-1.00f, -1.00f, -1.00f, //top right
			-1.00f, 1.00f, -1.00f, //top left
			};

	GLfloat tmpVolumeBoxcolor[] = {
	//front
			0.00f, 0.00f, 1.00f, //bottom left
			1.00f, 0.00f, 1.00f, //bottom right
			1.00f, 1.00f, 1.00f, //top right
			0.00f, 1.00f, 1.00f, //top left
			// back
			0.00f, 1.00f, 0.00f, //bottom left
			0.00f, 0.00f, 0.00f, //top right
			1.00f, 0.00f, 0.00f, //top right
			1.00f, 1.00f, 0.00f, //top left
			// top
			0.00f, 1.00f, 0.00f, //bottom right
			0.00f, 1.00f, 1.00f, //bottom left
			1.00f, 1.00f, 1.00f, //top left
			1.00f, 1.00f, 0.00f, //top right
			// bottom
			0.00f, 0.00f, 0.00f, //bottom right
			0.00f, 0.00f, 1.00f, //bottom left
			1.00f, 0.00f, 1.00f, //top left
			1.00f, 0.00f, 0.00f, //top right
			// right
			1.00f, 1.00f, 1.00f, //top left
			1.00f, 0.00f, 1.00f, //bottom right
			1.00f, 0.00f, 0.00f, //top right
			1.00f, 1.00f, 0.00f, //top left
			// left
			0.00f, 1.00f, 1.00f, //bottom left
			0.00f, 0.00f, 1.00f, //bottom right
			0.00f, 0.00f, 0.00f, //top right
			0.00f, 1.00f, 0.00f, //top left
			};

	GLushort tmpVolumeBoxTexCoords[] = {
			0, 0, 1, 0, 1, 1, 0, 1,
			1, 0, 0, 0, 0, 1, 1, 1,
			0, 0, 1, 0, 1, 1, 0, 1,
			1, 0, 0, 0, 0, 1, 1, 1,
			0, 0, 1, 0, 1, 1, 0, 1,
			1, 0, 0, 0, 0, 1, 1, 1
	};

	GLushort tmpVolumeBoxindices[] = { 3, 0, 1, 1, 2, 3, 7, 6, 5, 5, 4, 7, 8, 9,
			10, 10, 11, 8, 14, 13, 12, 12, 15, 14, 17, 18, 19, 19, 16, 17, 22,
			21, 20, 20, 23, 22 };

	memcpy(VolumeBoxvertices, tmpVolumeBoxvertices, sizeof(VolumeBoxvertices));
	memcpy(VolumeBoxcolor, tmpVolumeBoxcolor, sizeof(VolumeBoxcolor));
	memcpy(VolumeBoxTexCoords, tmpVolumeBoxTexCoords,sizeof(VolumeBoxTexCoords));
	memcpy(VolumeBoxindices, tmpVolumeBoxindices, sizeof(VolumeBoxindices));

}

Volume::~Volume() {
	glDeleteTextures(1, &intensityTexture);
	glDeleteBuffers(1, &modelVertexBuffer);
	glDeleteBuffers(1, &modelColorBuffer);
	glDeleteBuffers(1, &modelTexCordBuffer);
	glDeleteBuffers(1, &modelIndexBuffer);
	glDeleteBuffers(5, slicingQuadBuffer);

}

bool Volume::CreateTexture3D() {

	PFNGLTEXIMAGE3DOESPROC glTexImage3DOES;
	PFNGLTEXSUBIMAGE3DOESPROC glTexSubImage3DOES;
	PFNGLCOMPRESSEDTEXIMAGE3DOESPROC glCompressedTexImage3DOES;
	PFNGLCOMPRESSEDTEXSUBIMAGE3DOESPROC glCompressedTexSubImage3DOES;

	glTexImage3DOES = (PFNGLTEXIMAGE3DOESPROC) eglGetProcAddress("glTexImage3DOES");
	glTexSubImage3DOES = (PFNGLTEXSUBIMAGE3DOESPROC) eglGetProcAddress("glTexSubImage3DOES");
	glCompressedTexImage3DOES = (PFNGLCOMPRESSEDTEXIMAGE3DOESPROC) eglGetProcAddress("glCompressedTexImage3DOES");
	glCompressedTexSubImage3DOES = (PFNGLCOMPRESSEDTEXSUBIMAGE3DOESPROC) eglGetProcAddress("glCompressedTexSubImage3DOES");

	glGenTextures(1, &intensityTexture);

	int iTextureWidth;
	int iTextureHeight;
	int iTextureDepth;

	//If graphic card is not supporting non power of two textures
	iTextureWidth = GetNextPowerOfTwo(m_iWidth);
	iTextureHeight = GetNextPowerOfTwo(m_iHeight);
	iTextureDepth = GetNextPowerOfTwo(m_iDepth);

	textureRatio[0] = (float) m_iWidth / (float) iTextureWidth;
	textureRatio[1] = (float) m_iHeight / (float) iTextureHeight;
	textureRatio[2] = (float) m_iDepth / (float) iTextureDepth;


	if (GL_OES_texture_3D) {
		glBindTexture(GL_TEXTURE_3D_OES, intensityTexture);
		glTexImage3DOES(GL_TEXTURE_3D_OES, 0, GL_LUMINANCE, iTextureWidth,iTextureHeight, iTextureDepth, 0, GL_LUMINANCE,GL_UNSIGNED_BYTE, NULL);
		glTexSubImage3DOES(GL_TEXTURE_3D_OES, 0, 0, 0, 0, m_iWidth, m_iHeight,m_iDepth, GL_LUMINANCE, GL_UNSIGNED_BYTE, (void *) Get());
		glTexParameteri(GL_TEXTURE_3D_OES, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_3D_OES, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

		LOGI("Volume WIDTH, %d", GetWidth());
		LOGI("Volume HEIGHT, %d", GetHeight());
		LOGI("Volume DEPTH, %d", GetDepth());

		LOGI("Texture3D Loaded");

		setBoundingBox(m_iWidth, m_iHeight, m_iDepth);

		checkGlError("Texture3D");

		return true;

	} else {
		LOGI("Not supporting 3D Tex");
		return false;
	}

	return false;
}

void Volume::setBoundingBox(int vwidht, int vheight, int vdepth) {
	int max = vwidht;


	if (max < vheight) {
		max = vheight;

		if (max < vdepth) {
			max = vdepth;
		}
	}

	ratio[0] = (float) vwidht / max;
	ratio[1] = (float) vheight / max;
	ratio[2] = (float) vdepth / max;

	for (int a = 0; a < 72 - 2;) {
		VolumeBoxvertices[a] = VolumeBoxvertices[a] * ratio[0];
		VolumeBoxvertices[(a + 1)] = VolumeBoxvertices[(a + 1)] * ratio[1];
		VolumeBoxvertices[(a + 2)] = VolumeBoxvertices[(a + 2)] * ratio[2];
		a = a + 3;
	}

	LOGI("BoundingBox Scaled");

	this->CreateVolumeBox();
	this->CreateSlicingPlanes();
}

void Volume::CreateVolumeBox() {

	glGenBuffers(1, &modelVertexBuffer);
	glBindBuffer(GL_ARRAY_BUFFER, modelVertexBuffer);
	glBufferData(GL_ARRAY_BUFFER, sizeof(GLfloat) * 72, VolumeBoxvertices,
			GL_STATIC_DRAW);

	glGenBuffers(1, &modelColorBuffer);
	glBindBuffer(GL_ARRAY_BUFFER, modelColorBuffer);
	glBufferData(GL_ARRAY_BUFFER, sizeof(GLfloat) * 72, VolumeBoxcolor,
			GL_STATIC_DRAW);

	glGenBuffers(1, &modelTexCordBuffer);
	glBindBuffer(GL_ARRAY_BUFFER, modelTexCordBuffer);
	glBufferData(GL_ARRAY_BUFFER, sizeof(GLfloat) * 48, VolumeBoxTexCoords,
			GL_STATIC_DRAW);

	glGenBuffers(1, &modelIndexBuffer);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, modelIndexBuffer);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(GLushort) * 36,
			VolumeBoxindices, GL_STATIC_DRAW);

	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);

	LOGI("BoundingBox Binded");
}

void Volume::CreateSlicingPlanes()
{

	SlicingXquadVertices= new GLfloat[12];
	SlicingYquadVertices= new GLfloat[12];
	SlicingZquadVertices= new GLfloat[12];

	GLushort tmpquadTexCoords[] = { 0, 0, 0, 1, 1, 1, 1, 0 };
	memcpy(SlicingQuadTexCoords, tmpquadTexCoords, sizeof(SlicingQuadTexCoords));
	GLushort tmpquadindices[] = { 0, 3, 2, 0, 2, 1 };
	memcpy(SlicingQuadindices, tmpquadindices, sizeof(SlicingQuadindices));


	GLfloat tmpSlicingquadVertices[] = {-1.0f,-1.0f, 0.0f,-1.0f, 1.0f, 0.0f, 1.0f, 1.0f, 0.0f, 1.0f, -1.0f, 0.0f};

	for (int a = 0; a < 10;) {
		//width = volumedepth, height = volume-height
		SlicingXquadVertices[a] = tmpSlicingquadVertices[a] * ratio[2];
		SlicingXquadVertices[(a + 1)] = tmpSlicingquadVertices[(a + 1)] * ratio[1];

		// width = volumewidth, height = volumedepth
		SlicingYquadVertices[a] = tmpSlicingquadVertices[a] * ratio[0];
		SlicingYquadVertices[(a + 1)] = tmpSlicingquadVertices[(a + 1)] * ratio[2];

		//width = volumewidth, height = volumeHeight
		SlicingZquadVertices[a] = tmpSlicingquadVertices[a] * ratio[0];
		SlicingZquadVertices[(a + 1)] = tmpSlicingquadVertices[(a + 1)] * ratio[1];
		a = a + 3;
	}


	glGenBuffers(5, slicingQuadBuffer);
	glBindBuffer(GL_ARRAY_BUFFER, slicingQuadBuffer[0]);
	glBufferData(GL_ARRAY_BUFFER, sizeof(GLfloat) * 12, SlicingXquadVertices,GL_STATIC_DRAW);

	glBindBuffer(GL_ARRAY_BUFFER, slicingQuadBuffer[1]);
	glBufferData(GL_ARRAY_BUFFER, sizeof(GLfloat) * 12, SlicingYquadVertices,GL_STATIC_DRAW);

	glBindBuffer(GL_ARRAY_BUFFER, slicingQuadBuffer[2]);
	glBufferData(GL_ARRAY_BUFFER, sizeof(GLfloat) * 12, SlicingZquadVertices,GL_STATIC_DRAW);

	glBindBuffer(GL_ARRAY_BUFFER, slicingQuadBuffer[3]);
	glBufferData(GL_ARRAY_BUFFER, sizeof(GLushort) * 8, SlicingQuadTexCoords,GL_STATIC_DRAW);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, slicingQuadBuffer[4]);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(GLushort) * 6,SlicingQuadindices,GL_STATIC_DRAW);

	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);

	checkGlError("Slices");


}

bool Volume::LoadFile(std::string path) {

	FILE *fp = fopen(path.c_str(), "r");

	if (!fp) {
		std::cerr << "+ Error loading file." << std::endl << std::endl;
		return false;
	} else {
		char vcPath[1024];
		char *pFileName = NULL;

		/*GetFullPathName(strFilename.c_str(), 1024, vcPath, &pFileName);
		char vcDrive[1024], vcDirectory[1024], vcFilename[1024],
				vcExtension[1024];
		_splitpath_s(vcPath, vcDrive, vcDirectory, vcFilename, vcExtension);
		const std::string strAdditionalFilename = std::string(vcDrive)
				+ std::string(vcDirectory) + std::string(vcFilename)
				+ std::string(".ini");

		char vpSpacingX[1024], vpSpacingY[1024], vpSpacingZ[1024];
		GetPrivateProfileString("DatFile", "oldDat Spacing X", "1.0",
				vpSpacingX, 256, strAdditionalFilename.c_str());
		GetPrivateProfileString("DatFile", "oldDat Spacing Y", "1.0",
				vpSpacingY, 256, strAdditionalFilename.c_str());
		GetPrivateProfileString("DatFile", "oldDat Spacing Z", "1.0",
				vpSpacingZ, 256, strAdditionalFilename.c_str());*/

		unsigned short uWidth, uHeight, uDepth;
		fread(&uWidth, sizeof(unsigned short), 1, fp);
		fread(&uHeight, sizeof(unsigned short), 1, fp);
		fread(&uDepth, sizeof(unsigned short), 1, fp);

		m_iWidth = int(uWidth);
		m_iHeight = int(uHeight);
		m_iDepth = int(uDepth);

		const int iSlice = m_iWidth * m_iHeight;
		const int iSize = iSlice * m_iDepth;
		m_vecVoxels.resize(iSize);

		std::vector<unsigned short> vecData;
		vecData.resize(iSize);

		LOGI("SIZE: %d", vecData.size());

		std::cout << "SIZEBEFR: " << vecData.size();

		fread((void*) &(vecData.front()), sizeof(unsigned short),iSize,fp);
		fclose(fp);

		std::cout << "- File loaded." << std::endl;

		for (int k = 0; k < m_iDepth; k++) {
			for (int j = 0; j < m_iHeight; j++) {
				for (int i = 0; i < m_iWidth; i++) {
					//we convert the data to float values in an interval of [0..1]
					float fValue = std::min(1.0f,float(vecData[i + j *m_iWidth+ k * iSlice]) / 4095.0f);
					m_vecVoxels[(i + j * m_iWidth + k * iSlice)] =Voxel(static_cast<unsigned char>(fValue * 255.0f));}
						}
					}

		CreateTexture3D();
		return true;
	}
}

	void Volume::printGLString(const char *name, GLenum s) {
		const char *v = (const char *) glGetString(s);
		LOGI("GL %s = %s\n", name, v);
	}

	void Volume::checkGlError(const char* op) {
		for (GLint error = glGetError(); error; error = glGetError()) {
			LOGI("after %s() glError (0x%x)\n", op, error);
		}
	}

}

