/*
 * Volume.h
 *
 *  Created on: 17.08.2011
 *  Author: Manuel Hochmayr, e0627715@student.tuwien.ac.at
 *  Parts are taken for die VisLU framework. For more information see:
 *  https://www.cg.tuwien.ac.at/courses/Visualisierung/Angaben/VolVisFramework.zip
 */

#ifndef VOLUME_H_
#define VOLUME_H_


#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <vector>
#include <EGL/egl.h>
#include <GLES2/gl2.h>
#include <GLES2/gl2ext.h>

#include <android/log.h>
#include "cml/cml.h"
#include "Voxel.h"


#define  LOG_TAG    "VoRMD"
#define  LOGI(...)  __android_log_print(ANDROID_LOG_INFO,LOG_TAG,__VA_ARGS__)
#define  LOGE(...)  __android_log_print(ANDROID_LOG_ERROR,LOG_TAG,__VA_ARGS__)

namespace vormd {

class Volume {



public:

	GLuint modelVertexBuffer;
	GLuint modelColorBuffer;
	GLuint modelIndexBuffer;
	GLuint modelTexCordBuffer;

	GLfloat VolumeBoxvertices[72];
	GLfloat VolumeBoxcolor[72];
	GLushort VolumeBoxTexCoords[48];
	GLushort VolumeBoxindices[36];

	GLuint slicingQuadBuffer[5];
	//Slicing Quad X
	GLfloat *SlicingXquadVertices; // index 0
	GLfloat *SlicingYquadVertices;// index 1
	GLfloat *SlicingZquadVertices;// index 2

	GLushort SlicingQuadTexCoords[8];// index 3
	GLushort SlicingQuadindices[6];// index 4

	GLuint intensityTexture;
	std::vector<Voxel> m_vecVoxels;
	float textureRatio[3];
	int m_iWidth,m_iHeight,m_iDepth;
	cml::matrix44f_c volumeModelMatrix;


	Volume();

	virtual ~Volume();

	bool CreateTexture3D();

	bool LoadFile(std::string path);


	const Voxel & Get(const int iX, const int iY, const int iZ) const {
		return m_vecVoxels[iX + iY * m_iWidth + iZ * m_iWidth * m_iHeight];
	};

	const Voxel & Get(const int iIndex) const {
		return m_vecVoxels[iIndex];
	};

	const Voxel * Get() const {
		return &(m_vecVoxels.front());
	};

	const Voxel * GetVecVoxels() const {
			return &(m_vecVoxels[0]);
	};

	void Set(int iX, int iY, int iZ, float value) {
		m_vecVoxels[iX + iY * m_iWidth + iZ * m_iWidth * m_iHeight] = value;
	};

	const int GetWidth() const {
		return m_iWidth;
	};

	const int GetHeight() const {
		return m_iHeight;
	};

	const int GetDepth() const {
		return m_iDepth;
	};

	const int GetSize() const {
		return int(m_vecVoxels.size());
	};


private:
	/**
	 * Scales the bounding box to the size of the volume
	 */
	void setBoundingBox(int vwidht, int vheight, int vdepth);

	/**
	 * Sends the volume box to the GPU
	 */
	void CreateVolumeBox();

	/**
	 * Creats the slicing planes accroding to X Y and Z
	 */
	void CreateSlicingPlanes();
	void printGLString(const char *name, GLenum s);
	void checkGlError(const char* op);

	float ratio[3];



};

} /* namespace vormd */
#endif /* VOLUME_H_ */
