/*
 * Voxel.h
 *
 *  Created on: 17.08.2011
 *	Author: Manuel Hochmayr, e0627715@student.tuwien.ac.at
 *  Taken form die VisLU framework. For more information see:
 *  https://www.cg.tuwien.ac.at/courses/Visualisierung/Angaben/VolVisFramework.zip
 */

#ifndef VOXEL_H_
#define VOXEL_H_

namespace vormd {

class Voxel {
public:

	Voxel() {
		SetValue(0.0f);
	};

	Voxel(const Voxel & datOther) {
		m_fValue = datOther.m_fValue;
	};

	Voxel(const unsigned char fValue) {
		SetValue(fValue);
	};

	~Voxel() {
	};

	void SetValue(const unsigned char fValue) {
		m_fValue = fValue;
	};

	const unsigned char GetValue() const {
		return m_fValue;
	};

	const bool operator==(const Voxel &datOther) const {
		return (GetValue() == datOther.GetValue());
	};

	const bool operator!=(const Voxel &datOther) const {
		return !(*this == datOther);
	};

	const bool operator>(const Voxel &datOther) const {
		return GetValue() > datOther.GetValue();
	};

	const bool operator>=(const Voxel &datOther) const {
		return GetValue() >= datOther.GetValue();
	};

	const bool operator<(const Voxel &datOther) const {
		return GetValue() < datOther.GetValue();
	};

	const bool operator<=(const Voxel &datOther) const {
		return GetValue() <= datOther.GetValue();
	};

	const Voxel & operator+=(const Voxel & datOther) {
		m_fValue += datOther.m_fValue;
		return *this;
	};

	const Voxel & operator-=(const Voxel & datOther) {
		m_fValue -= datOther.m_fValue;
		return *this;
	};

	const Voxel & operator*=(const unsigned char & fOther) {
		m_fValue *= fOther;
		return *this;
	};

	const Voxel & operator/=(const unsigned char & fOther) {
		m_fValue /= fOther;
		return *this;
	};

	const Voxel operator+(const Voxel & datOther) const {
		Voxel voxNew = *this;
		voxNew += datOther;
		return voxNew;
	};

	const Voxel operator-(const Voxel & datOther) const {
		Voxel voxNew = *this;
		voxNew -= datOther;
		return voxNew;
	};

	const Voxel operator*(const unsigned char & fOther) const {
		Voxel voxNew = *this;
		voxNew *= fOther;
		return voxNew;
	};

	const Voxel operator/(const unsigned char & fOther) const {
		Voxel voxNew = *this;
		voxNew /= fOther;
		return voxNew;
	};

private:
	unsigned char m_fValue;
};



} /* namespace vormd */
#endif /* VOXEL_H_ */
