package com.vormd;

import android.content.Context;
import android.content.Intent;
import android.util.Log;
import android.view.GestureDetector;
import android.view.MotionEvent;
import android.view.ScaleGestureDetector;
import android.view.ScaleGestureDetector.OnScaleGestureListener;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnTouchListener;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemSelectedListener;
import android.widget.SeekBar;
import android.widget.SeekBar.OnSeekBarChangeListener;
import android.widget.Toast;

/**
 * Class detecting the touch input for rotation
 * @author Manuel
 *
 */
public class TouchInputDetector implements OnTouchListener {

	static final int MINDISTANCE = 3;
	static final String logTag = "SwipeDetector";
	private VoRMDActivity activity;
	private ScaleGestureDetector mScaleDetector;
	private float downX, downY, upX, upY, deltaX, deltaY;
	boolean performingUpdate;

	public TouchInputDetector(Context context) {
		this.activity = (VoRMDActivity) context;
		mScaleDetector = new ScaleGestureDetector(activity, new ScaleListener());
	}

	public boolean onTouch(View v, MotionEvent event) {
		mScaleDetector.onTouchEvent(event);

		if (!mScaleDetector.isInProgress()) {

			final int action = event.getAction();

			switch (action) {
				case MotionEvent.ACTION_DOWN: {
					downX = event.getX();
					downY = event.getY();
					VoRMDJNILIB.performingUpdate(true);
					break;
				}
	
				case MotionEvent.ACTION_UP: {
					VoRMDJNILIB.performingUpdate(false);
					break;
				}
	
				case MotionEvent.ACTION_MOVE: {
					upX = event.getX();
					upY = event.getY();
	
					deltaX = upX - downX;
					deltaY = upY - downY;
	
					if (Math.abs(deltaX) > MINDISTANCE || Math.abs(deltaY) > MINDISTANCE)
								downX = upX;
								downY = upY;
		
						// swipe horizontal?
						if (Math.abs(deltaX) > MINDISTANCE) { 
							// left or right
							if (deltaX > 0) {
								VoRMDJNILIB.rotateLeft();
							}
							if (deltaX < 0) {
								VoRMDJNILIB.rotateRight();
							}
						} 
						if (Math.abs(deltaY) > MINDISTANCE) {
							// top or down
							if (deltaY > 0) {
								VoRMDJNILIB.rotateDown();
							}
							if (deltaY < 0) {
								VoRMDJNILIB.rotateUp();
							}
						}
					}
		
					break;
				}
			}
		

		return true;
	}

	
	


	
	
	
	

}
