package com.vormd;


import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Matrix;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.SeekBar;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.SeekBar.OnSeekBarChangeListener;

/**
 * Class representing the Userinterface. It handels all the input via the menu, buttons and slider.
 * @author Manuel Hochmayr, 0627715@student.tuwien.ac.at
 *
 */
public class UserInterface implements OnClickListener,OnSeekBarChangeListener {
	
	private VoRMDActivity context;
	private TouchInputDetector activitySwipeDetector;
	private Settings settings;
	private Intent myIntent; 
	private Menu menu;
	
	private TextView infotext;
	private TextView dz;
	private SeekBar spinner;
	private Button dzInc;
	private Button dzDec;
	private Button coordinates;
	private View uioverlay;
	private int buttonPlusId;
	private int buttonMinusId;
	private int buttonCoordId;
	private boolean fromSlicing;
	private boolean loadDat;
	private boolean loadPNG;
	
	/**
	 * Constructor for the UI. 
	 * @param context the activity
	 * @param activitySwipeDetector SwipDetector
	 * @param settings Setting class
	 */
	public UserInterface(Context context,TouchInputDetector activitySwipeDetector,Settings settings){
		this.context = (VoRMDActivity) context;
		this.activitySwipeDetector = activitySwipeDetector;
		this.settings = settings;
		
		this.addUiOverlay();
		
		if(settings.getRenderMode() == Settings.SLICING){
			fromSlicing = true;
			this.showSpinner();
		}else{
			fromSlicing = false;
		}
	}
	/**
	 * Adds the UI as overlay over OpenGl context. Furthermore it gets all the IDs of the UI elements form R
	 */
	private void addUiOverlay() {
		LayoutInflater inflater = context.getLayoutInflater();
		uioverlay = inflater.inflate(R.layout.main, null);
		context.getWindow().addContentView(
				uioverlay,
				new ViewGroup.LayoutParams(ViewGroup.LayoutParams.FILL_PARENT,
						ViewGroup.LayoutParams.FILL_PARENT));
		infotext = (TextView) context.findViewById(R.id.infotext);
		dz = (TextView) context.findViewById(R.id.dz_field);
		spinner = (SeekBar) context.findViewById(R.id.sb_slicing);
		dzInc = (Button) context.findViewById(R.id.incDZ);
		dzDec = (Button) context.findViewById(R.id.decDZ);
		coordinates = (Button) context.findViewById(R.id.coord_slicing);
		
	
		spinner.setOnSeekBarChangeListener(this);
		dzInc.setOnClickListener(this);
		dzDec.setOnClickListener(this);
		coordinates.setOnClickListener(this);
		
		dz.setText("dz: " + settings.getDz());
		buttonPlusId = dzInc.getId();
		buttonMinusId = dzDec.getId();
		buttonCoordId = coordinates.getId();
		
		coordinates.setText("X");
		loadDat = false;
		loadPNG = false;
		
		
	}
	/**
	 * Handler for the submenu of the Shader menu
	 * @param men
	 */
	public void onCreateOptionsMenu(Menu men){
		this.menu = men;
		
		if(settings.getRenderMode() == Settings.MIP){
			this.menu.findItem(R.id.shad_mip).setChecked(true);
		}
		
		if(settings.getRenderMode() == Settings.AVARAGING){
			this.menu.findItem(R.id.shad_avg).setChecked(true);
		}
		
		if(settings.getRenderMode() == Settings.COMPOSITING){
			this.menu.findItem(R.id.shad_comp).setChecked(true);
		}
		
		if(settings.getRenderMode() == Settings.SLICING){
			this.menu.findItem(R.id.shad_slicing).setChecked(true);
		}
		
		
		this.menu.findItem(R.id.men_adap).setChecked(settings.isAdaptiv());
		this.menu.findItem(R.id.men_fps).setChecked(settings.isShowFPS());
		
		
			
	}
	
	
	/**
	 * Handler for the menu itself
	 * @param item the menu item pressed
	 * @return
	 */
	public boolean onOptionsItemSelected(MenuItem item) {
		
		switch(item.getItemId()){
			
		case R.id.loadVolume:
			myIntent = new Intent(context,FileChooser.class);
			loadDat = true;
			context.startActivityForResult(myIntent, 0);
			Log.i(VoRMDActivity.TAG,"LOADVOL");
			
			
			return true;
			
		case R.id.loadTransfer:
			myIntent = new Intent(context, FileChooser.class);
			loadPNG = true;
			context.startActivityForResult(myIntent, 0);
			Log.i(VoRMDActivity.TAG,"LOADTR");
			
			return true;
			
		case R.id.shad_mip:
			Log.i(VoRMDActivity.TAG,"MIP");
			context.getSettings().setRenderMode(Settings.MIP);
			this.menu.findItem(R.id.shad_mip).setChecked(true);
			
			if(fromSlicing){
				this.hideSpinner();
				fromSlicing = false;
			}
			
			return true;
		case R.id.shad_avg:
			Log.i(VoRMDActivity.TAG,"AVG");
			context.getSettings().setRenderMode(Settings.AVARAGING);
			this.menu.findItem(R.id.shad_avg).setChecked(true);
			
			if(fromSlicing){
				this.hideSpinner();
				fromSlicing = false;
			}
			
			return true;
		case R.id.shad_comp:
			Log.i(VoRMDActivity.TAG,"COMP");
			context.getSettings().setRenderMode(Settings.COMPOSITING);
			this.menu.findItem(R.id.shad_comp).setChecked(true);
			
			if(fromSlicing){
				this.hideSpinner();
				fromSlicing = false;
			}
			
			return true;
		case R.id.shad_slicing:
			Log.i(VoRMDActivity.TAG,"SLICING");
			context.getSettings().setRenderMode(Settings.SLICING);
			this.menu.findItem(R.id.shad_slicing).setChecked(true);
			
			
			this.showSpinner();
			fromSlicing = true;
			
			
			return true;
		case R.id.men_adap:
			if(settings.isAdaptiv()){
				settings.setAdaptiv(false);				
			}else{
				settings.setAdaptiv(true);
			}
			this.menu.findItem(R.id.men_adap).setChecked(settings.isAdaptiv());
			VoRMDJNILIB.setAdaptiv(settings.isAdaptiv());
			return true;
			
		case R.id.men_fps:
			if(settings.isShowFPS()){
				settings.setShowFPS(false);				
			}else{
				settings.setShowFPS(true);
			}
			this.hideShowFPS();
			this.menu.findItem(R.id.men_fps).setChecked(settings.isShowFPS());
			Log.i(VoRMDActivity.TAG,"FPS/ONOFF");
			return true;
		
		
		case R.id.quit:
			VoRMDJNILIB.quitApp();
			context.finish();
			return true;	
	}
		
		return true;
	}
	
	 /**
	  * Method which will be called after returning form the File choose activity. It gets a result from the activity.
	  * @param requestCode 
	  * @param resultCode
	  * @param data
	  */
	public void onActivityResult(int requestCode, int resultCode, Intent data) {     
	  switch(requestCode) { 
	    case (0) : { 
	      if (resultCode == VoRMDActivity.RESULT_OK){ 
	    	  
	    	  if(loadDat){
	    		  if(data.getStringExtra(FileChooser.FILEID).endsWith(".dat") && !data.getStringExtra(FileChooser.FILEID).startsWith(".-")){
			    	  context.getSettings().setVolumeFilename(data.getStringExtra(FileChooser.FILEID));
			    	  context.getSettings().setAssetFile(false);
			    	  Log.i(VoRMDActivity.TAG,context.getSettings().getVolumeFilename());
	    		  }else{
	    			  Toast.makeText(context, "Could not load PNG image.Not known image format.", Toast.LENGTH_LONG).show();
	    		  }
	    		  loadDat = false;
	    	  }
	    	 if(loadPNG){  
		    	 if (data.getStringExtra(FileChooser.FILEID).endsWith(".png") && !data.getStringExtra(FileChooser.FILEID).startsWith(".-")){
			    		  BitmapFactory.Options options = new BitmapFactory.Options();
			    		  options.inScaled = false;
			    		  Bitmap bm = BitmapFactory.decodeFile(data.getStringExtra(FileChooser.FILEID));
			    		  context.getSettings().setUsingStandardTransfer(false);
			    		  context.getSettings().setTransferFunctionBMP(resizeBmp(bm,bm.getHeight(),4096));
		    	  }else{
		    		  Toast.makeText(context, "Could not load PNG image.Not known image format.", Toast.LENGTH_LONG).show();
		    	  }
		    	 loadPNG = false;
	    	 } 
	      break; 
	      }
	    } 
	  } 
	}
	
	/**
	 * Method for changing the UI text
	 * @param text
	 */
	public void uiChanged(String text) {
		infotext.setText(text);
	}

	/**
	 * Method to show the spinner and hide all other UI elements
	 */
	public void showSpinner() {
		spinner.setVisibility(View.VISIBLE);
		coordinates.setVisibility(View.VISIBLE);
		dzInc.setVisibility(View.INVISIBLE);
		dzDec.setVisibility(View.INVISIBLE);
		dz.setVisibility(View.INVISIBLE);
		
	}

	/**
	 * Method to hide the spinner and show the other UI elements. 
	 */
	public void hideSpinner() {
		spinner.setVisibility(View.INVISIBLE);
		coordinates.setVisibility(View.INVISIBLE);
		dzInc.setVisibility(View.VISIBLE);
		dzDec.setVisibility(View.VISIBLE);
		dz.setVisibility(View.VISIBLE);
	}
	
	/**
	 * Method to change the spinners size and the spinner mode according to the coordinate choosen by the user
	 */
	public void setSpinner(){
		if(coordinates.getText() == "X"){
			spinner.setMax(VoRMDJNILIB.getVolumeWidth());
		}
		
		else if(coordinates.getText() == "Y"){
			spinner.setMax(VoRMDJNILIB.getVolumeHeight());
		}
		
		else if(coordinates.getText() == "Z"){
			spinner.setMax(VoRMDJNILIB.getVolumeDepth());
		}
	}


	/**
	 * Method to show and hide the FPS element
	 */
	public void hideShowFPS() {
		if (infotext.getVisibility() == View.VISIBLE)
			infotext.setVisibility(View.INVISIBLE);
		else if (infotext.getVisibility() == View.INVISIBLE)
			infotext.setVisibility(View.VISIBLE);
	}
	
	/**
	 * Method which will be called when when a button was pressed. It changes dz and the slicing mode 
	 */
	@Override
	public void onClick(View arg0) {
		
		
		
		if(arg0.getId() == this.buttonPlusId){
			float tmpdz = settings.getDz();
			
			if(tmpdz < 0.1f-0.001){
				tmpdz += 0.001;
				settings.setDz(tmpdz);
				VoRMDJNILIB.setDz(settings.getDz());
				int ttmpz = (int) (tmpdz*1000);
				tmpdz = (float)ttmpz/1000.0f;
				dz.setText("dz: " + tmpdz);
				//Toast.makeText(context, "dz: "+ settings.getDz() , Toast.LENGTH_SHORT).show();
			}
			
		}
	
		if(arg0.getId() == this.buttonMinusId){
			
			float tmpdz = settings.getDz();
			
			if(tmpdz > 0.001 + 0.001f){
				tmpdz -= 0.001;
				settings.setDz(tmpdz);
				VoRMDJNILIB.setDz(settings.getDz());
				int ttmpz = (int) (tmpdz*1000);
				tmpdz = (float)ttmpz/1000.0f;
				dz.setText("dz: " + tmpdz);
				//Toast.makeText(context, "dz: "+ settings.getDz(),Toast.LENGTH_SHORT).show();
			}
			
		}
		
		if(arg0.getId() == buttonCoordId){
			
			if(coordinates.getText() == "X"){
				coordinates.setText("Y");
				VoRMDJNILIB.setSlicingMode(1);
				//spinner.setMax(VoRMDJNILIB.getVolumeWidth());
			}
			
			else if(coordinates.getText() == "Y"){
				coordinates.setText("Z");
				VoRMDJNILIB.setSlicingMode(2);
				//spinner.setMax(VoRMDJNILIB.getVolumeHeight());
			}
			
			else if(coordinates.getText() == "Z"){
				coordinates.setText("X");
				VoRMDJNILIB.setSlicingMode(0);
				//spinner.setMax(VoRMDJNILIB.getVolumeDepth());
			}
			
			this.setSpinner();
			
		}
		
		
	}
	
	/**
	 * Method for the seekBar to change the Z depth value of the slices - For
	 * slicing through the 3D texture
	 */
	@Override
	public void onProgressChanged(SeekBar arg0, int arg1, boolean arg2) {
		Log.i(VoRMDActivity.TAG, "Arg: " + arg0.getProgress());
		VoRMDJNILIB.setZCoord(arg0.getProgress());

	}

	/**
	 * Method for the seekBar
	 */
	@Override
	public void onStartTrackingTouch(SeekBar arg0) {
		// TODO Auto-generated method stub

	}

	/**
	 * Method for the seekBar
	 */
	@Override
	public void onStopTrackingTouch(SeekBar arg0) {
		// TODO Auto-generated method stub

	}
	
	public Bitmap resizeBmp(Bitmap bmp, int nHeight, int nWidth) {
		int width = bmp.getWidth();
		int height = bmp.getHeight();
		float scaleWidth = ((float) nWidth) / width;
		float scaleHeight = ((float) nHeight) / height;

		Matrix matrix = new Matrix();

		matrix.postScale(scaleWidth, scaleHeight);
		Bitmap resizedBitmap = Bitmap.createBitmap(bmp, 0, 0, width, height, matrix, false);

		return resizedBitmap;

	}



}
