package com.vormd;

import com.vormd.GLES2View;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;

/**
 * VoRMD - Volume rendering on mobile devices. 
 * Framework to load and render volumes on a mobile device with Android 2.3.
 * The device needs 3D texture support and OpenGL ES 2.0
 * This class represents the main activity which will be started when the users starts the program. 
 * It creats the OpenGL ES 2.0 View. 
 * @author Manuel Hochmayr,Bsc -  e0627715@student.tuwien.ac.at
 * Icons from: http://www.productivedreams.com/20-semi-transparent-glossy-icons/ - Icons are free to use
 */
public class VoRMDActivity extends Activity{

	protected static String TAG = "VORMD";
	public static final Handler uiHandler = new Handler();

	private GLES2View gles20view;

	private UserInterface UIinterface;
	private Settings settings;

	boolean showFPS;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		settings = new Settings();
		
		gles20view = new GLES2View(this);
		setContentView(gles20view);
		
		TouchInputDetector activitySwipeDetector = new TouchInputDetector(this);
		
		gles20view.setOnTouchListener(activitySwipeDetector);
		UIinterface = new UserInterface(this,activitySwipeDetector,settings);
		

	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.layout.menu, menu);
		this.UIinterface.onCreateOptionsMenu(menu);
		return true;
	}
	
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		this.UIinterface.onOptionsItemSelected(item);
		return true;
	}
	
	
	@Override 
	public void onActivityResult(int requestCode, int resultCode, Intent data) {     
	  super.onActivityResult(requestCode, resultCode, data); 
	  this.UIinterface.onActivityResult(requestCode, resultCode, data);
	  
	}
	

	@Override
	protected void onPause() {
		super.onPause();
		gles20view.onPause();
	}

	@Override
	protected void onResume() {
		super.onResume();
		gles20view.onResume();
	}
	
	
	public UserInterface getUI()
	{
		return this.UIinterface;
	}
	
	public Settings getSettings()
	{
		return this.settings;
	}

	
	

	

}